@@ -1,3 +1,6 @@
+#
+#HTML::Merge - Embedded HTML/SQL/Perl system by Raz Information Systems.
+#
 use strict;
 
 open(O, ">samples/samples.html") || die $!;
@@ -31,7 +34,7 @@ foreach (glob("samples/*.html")) {
 }
 
 foreach (sort {$hash{$a} cmp $hash{$b}} keys %hash) {
-	print O qq!\t<LI><A HREF="<\$RMERGE>?template=$_">$hash{$_}</A>\n!;
+	print O qq!\t<LI><A HREF="<\$RMERGE>?template=$_">$hash{$_}</A>&nbsp;-&nbsp;\n!;
 	print O qq!\t\t<\$RSOURCE.'$_'>view source</\$RSOURCE>\n!;
 }
 
